'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

exports.default = function (t) {
	return {
		ExpressionStatement(path, state) {
			var expression = path.node.expression;
			if (!(0, _utils.isIdAssignment)(t, expression) || !(0, _utils.isMethodCall)(t, expression.right, 'reduce')) {
				return;
			}

			var assignee = expression.left;

			var acc = writeReduceForLoop(t, path, path.get('expression.right'));
			path.replaceWith(t.assignmentExpression('=', assignee, acc));
		},

		VariableDeclaration(path, state) {
			if (path.node.declarations.length !== 1) return;

			if (!(0, _utils.isMethodCall)(t, path.node.declarations[0].init, 'reduce')) {
				return;
			}

			var reducePath = path.get('declarations.0.init');
			var acc = writeReduceForLoop(t, path, reducePath);
			reducePath.replaceWith(acc);
		}
	};
};

var _utils = require('../utils');

// Returns the identifier the result is stored in.
function writeReduceForLoop(t, path, reducePath) {
	var reduceCall = reducePath.node;
	var array = (0, _utils.defineIdIfNeeded)(t, reducePath.get('callee.object'), path);
	var func = (0, _utils.extractDynamicFuncIfNeeded)(t, reducePath, path);
	var acc = path.scope.generateUidIdentifier('acc');
	var i = path.scope.generateUidIdentifier('i');

	// Initialize the accumulator.
	// let acc = arguments[1] || arr[0];
	var initVal = reduceCall.arguments[1] || t.memberExpression(array, t.numericLiteral(0), true);
	path.insertBefore(t.VariableDeclaration('let', [t.VariableDeclarator(acc, initVal)]));

	// Write the for loop.
	var newReduceCall = t.callExpression(func, [acc, t.memberExpression(array, i, true), i, array]);
	var forBody = t.blockStatement([t.expressionStatement(t.assignmentExpression('=', acc, newReduceCall))]);
	var initI = reduceCall.arguments[1] ? undefined : t.numericLiteral(1);
	path.insertBefore((0, _utils.basicArrayForLoop)(t, i, array, forBody, initI));

	return acc;
}

/**
 * Returns a visitor that rewrites array.reduce() calls as a for loop.
 *
 * Only supports assignments and single declarations:
 *     results = arr.reduce(f); // assignment
 *     const results = arr.reduce(f); // single declaration
 */